/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package it.tidalwave.semantic.viewer.impl;

import javax.annotation.Nonnull;
import java.util.HashSet;
import java.util.Set;
import org.netbeans.platformx.eventbus.api.EventBus;
import org.netbeans.platformx.eventbus.api.EventBusAdapter;
import org.netbeans.platformx.eventbus.api.EventBusListener;
import it.tidalwave.semantic.other.Entity;
import it.tidalwave.semantic.other.Relation;
import it.tidalwave.semantic.other.Relationable;
import it.tidalwave.semantic.graph.VisualSemanticController;
import it.tidalwave.semantic.selection.RelationableSelection;
import it.tidalwave.semantic.viewer.SemanticViewer;

/**
 *
 * @author fritz
 */
public class SemanticViewerControllerImpl
  {
    @Nonnull
    private final SemanticViewer.View view;

    @Nonnull
    private final VisualSemanticController graphController;

    /***************************************************************************
     *
     * Notifies when a new {@link RelationableSelection} has been selected.
     *
     **************************************************************************/
    private final EventBusListener<RelationableSelection> selectionListener = new EventBusAdapter<RelationableSelection>()
      {
        @Override
        public void notify (final RelationableSelection selection)
          {
//            logger.finest("selectionListener.notify(%s)", dataObject);

            if (selection != null)
              {
                setSelection(selection);
              }
          }
      };

    /***************************************************************************
     *
     *
     **************************************************************************/
    public SemanticViewerControllerImpl (@Nonnull final SemanticViewer.View view,
                                         @Nonnull final VisualSemanticController graphController)
      {
        this.view = view;
        this.graphController = graphController;
        EventBus.getDefault().subscribe(RelationableSelection.class, selectionListener);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    private void setSelection (@Nonnull final RelationableSelection selection)
      {
        final Set<Entity> entities = new HashSet<Entity>();

        for (final Relationable relationable : selection.getContents())
          {
            if (relationable instanceof Entity)
              {
                entities.add((Entity)relationable);

                for (final Relation relation : relationable.findRelations())
                  {
                    entities.add((relation.getSubject() == relationable) ? (Entity)relation.getObject() : (Entity)relation.getSubject());
                  }
              }
          }

        graphController.clean();
        graphController.addEntities(entities);
      }
  }
